#include "CGDVncClient.h"

/*******************************************************************************/

CGDVncClient::CGDVncClient(int maxBlinks, int maxSamples, int maxFixations, std::string logFolder):
	mLog(logFolder)
{
	mBlinks = CGDBlinksQueuePtr(new CGDEntriesQueue<GDBlink>(maxBlinks));
	mLeftSamples = CGDSSampleQueuePtr(new CGDEntriesQueue<GDSingleSample>(maxSamples));
	mRightSamples = CGDSSampleQueuePtr(new CGDEntriesQueue<GDSingleSample>(maxSamples));
	mFixations = CGDFixationsQueuePtr(new CGDEntriesQueue<GDFixation>(maxFixations));

	initLogFile();
}

/*******************************************************************************/

CGDVncClient::~CGDVncClient()
{
	if (mBlinks)
	{
		mBlinks->clear();
		mBlinks.reset();
	}

	if (mLeftSamples)
	{
		mLeftSamples->clear();
		mLeftSamples.reset();
	}

	if (mRightSamples)
	{
		mRightSamples->clear();
		mRightSamples.reset();
	}

	if (mFixations)
	{
		mFixations->clear();
		mFixations.reset();
	}

	if (mLog.isReady())
	{
		mLog.close();
	}
}

/*******************************************************************************/

void CGDVncClient::initLogFile()
{
	if (mLog.isReady())
		return;

	ostringstream fileName;
	fileName << "GazeLog_" << getLogFileMarker() << "_at" << TimeStamp() << ".log";

	try
	{
		mLog.open(fileName.str(), m_id);
	}
	catch(std::exception ex){ }
}

/*******************************************************************************/

std::string CGDVncClient::getLogFileMarker()
{
	stringstream ss;
	ss << m_id;
	
	return ((ss.tellp() < 1) ? "[none]" : ss.str());
}

/*******************************************************************************/

std::string CGDVncClient::nickname()
{
	return mNickname.value();
}

/*******************************************************************************/

std::string CGDVncClient::description()
{
	return mDescription.value();
}

/*******************************************************************************/

BOOL CGDVncClient::isRegistered()
{
	return mRegistered;
}

/*******************************************************************************/

template<typename T>
BOOL CGDVncClient::sendGDMessage(CARD8 msgType, vncClientId id, int structSize, std::list<T> entries)
{
	if (entries.size() < 1)
		return false;

	gdServerGDHdrMsg gdHdrMsg;
	gdHdrMsg.id = Swap16IfLE(id);
	gdHdrMsg.count = Swap16IfLE(entries.size());

	omni_mutex_lock l(m_sendUpdateLock);

	//send header with the number of following entries structs
	if (!SendRFBMsg(msgType, (BYTE*) &gdHdrMsg, sz_gdServerGDHdrMsg))
	{
		return false;
	}

	//send structs
	for(std::list<T>::iterator it = entries.begin(); it != entries.end(); it++)
	{
		if (!m_socket->SendQueued((char *) &(*it), structSize))
		{
			Kill();
			return false;
		}
	}

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::sendTextMessage(CARD8 msgType, vncClientId id, std::string text)
{
	if (text.length() < 1)
		return false;

	if (text.length() > 256)
		text = text.substr(0, 256);

	gdServerTextHdrMsg gdTextHdrMsg;
	gdTextHdrMsg.id = Swap16IfLE(id);
	gdTextHdrMsg.length = text.length();
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send header with the number of following characters
	if (!SendRFBMsg(msgType, (BYTE*) &gdTextHdrMsg, sz_gdServerTextHdrMsg))
	{
		return false;
	}

	if (!m_socket->SendQueued((char *) text.c_str(), text.length()))
	{
		Kill();
		return false;
	}

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::sendIdMessage(CARD8 msgType, vncClientId id)
{
	gdServerIdMsg gdIdMsg;
	gdIdMsg.id = Swap16IfLE(id);
	
	omni_mutex_lock l(m_sendUpdateLock);

	//send header with the number of following characters
	if (!SendRFBMsg(msgType, (BYTE*) &gdIdMsg, sz_gdServerIdMsg))
	{
		return false;
	}

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::sendGDBlink(vncClientId id, std::list<gdBlink> blinks)
{
	return sendGDMessage(gdServerGazeBlink, id, sz_gdBlink, blinks);
}

/*******************************************************************************/

BOOL CGDVncClient::sendGDSingleSample(vncClientId id, std::list<gdSingleSample> samples)
{
	return sendGDMessage(gdServerGazeSample, id, sz_gdSingleSample, samples);
}

/*******************************************************************************/

BOOL CGDVncClient::sendGDFixation(vncClientId id, std::list<gdFixation> fixations)
{
	return sendGDMessage(gdServerGazeFixation, id, sz_gdFixation, fixations);
}

/*******************************************************************************/

BOOL CGDVncClient::sendNickname(vncClientId id, std::string nn)
{
	return sendTextMessage(gdServerNickname, id, nn);
}

/*******************************************************************************/

BOOL CGDVncClient::sendDescription(vncClientId id, std::string dscr)
{
	return sendTextMessage(gdServerDescription, id, dscr);
}

/*******************************************************************************/

BOOL CGDVncClient::sendTextMessage(vncClientId id, std::string txt)
{
	return sendTextMessage(gdServerTxtMessage, id, txt);
}

/*******************************************************************************/

BOOL CGDVncClient::sendRegRequest(vncClientId id, std::string nickname, std::string description)
{
	return sendIdMessage(gdServerReg, id) && 
		sendNickname(id, nickname) &&
		sendDescription(id, description);
}

/*******************************************************************************/

BOOL CGDVncClient::sendUnRegRequest(vncClientId id)
{
	return sendIdMessage(gdServerUnReg, id);
}

/*******************************************************************************/

BOOL CGDVncClient::sendPoke(vncClientId id)
{
	return sendIdMessage(gdServerPoke, id);
}

/*******************************************************************************/

BOOL CGDVncClient::receiveGDSingleSample(std::list<gdSingleSample> samples)
{
	string err;
	
	for(std::list<gdSingleSample>::iterator it; it != samples.end(); it++)
	{
		err = "";

		GDSingleSample* smpl = new GDSingleSample(*it);

		switch(smpl->eye())
		{
		case GDE_LEFT: //left eye
			mLeftSamples->add(smpl);
			break;
		case GDE_RIGHT: //right eye
			mRightSamples->add(smpl);
			break;
		default:
			err = "Unrecognized 'Eye'-field value";
			break;
		}

		mLog.log(LRT_GAZEDATA, smpl->toString(), err);
	}

	//@todo Send last sample to other clients

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveGDBlink(std::list<gdBlink> blinks)
{
	string err;
	
	for(std::list<gdBlink>::iterator it; it != blinks.end(); it++)
	{
		err = "";

		GDBlink *blnk = new GDBlink(*it);

		mBlinks->add(blnk);

		mLog.log(LRT_GAZEDATA, blnk->toString(), err);
	}

	//@todo Send last blink to other clients

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveGDFixation(std::list<gdFixation> fixations)
{
	string err;
	
	for(std::list<gdFixation>::iterator it; it != fixations.end(); it++)
	{
		err = "";

		GDFixation *fxn = new GDFixation(*it);

		mFixations->add(fxn);

		mLog.log(LRT_GAZEDATA, fxn->toString(), err);
	}

	//@todo Send last fixation to other clients

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveNickname(std::string nn)
{
	mNickname = nn;

	mLog.log(LRT_SETNICKNAME, mNickname.value());

	//@todo Send new nickname to other clients

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveDescription(std::string nd)
{
	mDescription = nd;

	mLog.log(LRT_SETDESCRIPT, mDescription.value());
	
	//@todo Send new nickname to other clients

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveTextMessage(std::string msg)
{
	mLog.log(LRT_MESSAGE, msg);
	
	//@todo Send message to other clients (?)

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveStartTracking()
{
	mRecStart = TimeStamp();

	mLog.log(LRT_STARTTRACK, "Tracking restarted");
	mLog.setRecordStartToken(mRecStart);

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receiveUnRegRequestClient()
{
	if (isRegistered())
	{
		mRegistered = false;
		mLog.log(LRT_UNREGCLIENT, "Unregister client");

		//@todo distribute unregister message
	}

	return true;
}

/*******************************************************************************/

BOOL CGDVncClient::receivePoke()
{
	if (isRegistered())
	{
		mRegistered = false;
		mLog.log(LRT_UNREGCLIENT, "Unregister client");

		//@todo distribute unregister message
	}

	return true;
}

/*******************************************************************************/
/*
#pragma region RECEIVING data
	BOOL ReceiveRegRequest(std::string, std::string);
	BOOL ReceivePoke();
#pragma endregion
*/

//TODO:
/**
Receiving packs of entries:
* receive several of entries
* try to add all of them to the queue
* distribute the last non-error entry
*/