#include "CGazeDataClient.h"

CGazeDataClient::CGazeDataClient(int blinkLength, int fixationLength, int sampleLength, short id, std::string logFolder)
{
	m_blinkQueueLength = blinkLength;
	m_fixationQueueLength = fixationLength;
	m_sampleQueueLength = sampleLength;

	m_blinks = etiGazeBlinkVector();
	m_fixations = etiGazeFixationVector();
	m_leftEyeSamples = etiGazeSampleVector();
	m_rightEyeSamples = etiGazeSampleVector();

	InitializeLogFiles(id, logFolder);
}

CGazeDataClient::~CGazeDataClient(void)
{
	DeInitializeLogFiles();
	ClearGazeDataVectors();
}

void CGazeDataClient::InitializeLogFiles(short clientId, std::string logFolder)
{
		etiTimeStamp initTime = GetETITimeStamp();
		std::ostringstream fileNamePreffix;

		fileNamePreffix << logFolder << "\\" << "Client" << clientId << "_";
		fileNamePreffix << initTime.Milliseconds << "_";

		m_blinksLog = CGazeDataLoggerPtr(new CGazeDataLogger(fileNamePreffix.str() + "blinks.txt"));
		m_fixationsLog = CGazeDataLoggerPtr(new CGazeDataLogger(fileNamePreffix.str() + "fixations.txt"));
		m_singleSamplesLog = CGazeDataLoggerPtr(new CGazeDataLogger(fileNamePreffix.str() + "samples.txt"));

		//blinks log file header
		m_blinksLog->Print("LRT_GAZEDATA");
		m_blinksLog->Print("GDT_BLINK");
		m_blinksLog->Print("Timestamp(ms)");
		m_blinksLog->Print("OrdinalNumber");
		m_blinksLog->Print("Duration");
		m_blinksLog->Print("Eye");
		m_blinksLog->Flush();
		
		//fixations log file header
		m_fixationsLog->Print("LRT_GAZEDATA");
		m_fixationsLog->Print("GDT_FIXATION");
		m_fixationsLog->Print("Timestamp(ms)");
		m_fixationsLog->Print("OrdinalNumber");
		m_fixationsLog->Print("Duration");
		m_fixationsLog->Print("X");
		m_fixationsLog->Print("Y");
		m_fixationsLog->Flush();

		//samples log file header
		m_singleSamplesLog->Print("LRT_GAZEDATA");
		m_singleSamplesLog->Print("GDT_SINGLESAMPLE");
		m_singleSamplesLog->Print("Timestamp(ms)");
		m_singleSamplesLog->Print("OrdinalNumber");
		m_singleSamplesLog->Print("X");
		m_singleSamplesLog->Print("Y");
		m_singleSamplesLog->Print("Eye");
		m_singleSamplesLog->Print("Pupil");
		m_singleSamplesLog->Flush();
}

void CGazeDataClient::DeInitializeLogFiles()
{
	m_blinksLog->Close();

	m_fixationsLog->Close();

	m_singleSamplesLog->Close();
}

void CGazeDataClient::ClearGazeDataVectors()
{
	m_blinks.clear();
	m_fixations.clear();
	m_leftEyeSamples.clear();
	m_rightEyeSamples.clear();
}

void CGazeDataClient::TrackingStarted()
{
	etiTimeStamp newRecordStart = GetETITimeStamp();

	m_recordStart.Year = newRecordStart.Year;
	m_recordStart.Days = newRecordStart.Days;
	m_recordStart.Milliseconds = newRecordStart.Milliseconds;

	//if record restarted - delete old data
	ClearGazeDataVectors();
}

void CGazeDataClient::AddBlink(CGazeDataEntryBlinkPtr etiItem)
{
	while (m_blinks.size() >= m_blinkQueueLength &&
		m_blinks.size() > 0)
		m_blinks.erase(m_blinks.begin());

	m_blinks.push_back(etiItem);

	//log blinks
	m_blinksLog->Print(LRT_GAZEDATA);
	m_blinksLog->Print(GDT_BLINK);
	m_blinksLog->Print(etiItem->Time());
	m_blinksLog->Print(etiItem->Number());
	m_blinksLog->Print(etiItem->Duration());
	m_blinksLog->Print(etiItem->Eye());
	m_blinksLog->Flush();
}

void CGazeDataClient::AddFixation(CGazeDataEntryFixationPtr etiItem)
{
	while (m_fixations.size() >= m_fixationQueueLength &&
		m_fixations.size() > 0)
		m_fixations.erase(m_fixations.begin());

	m_fixations.push_back(etiItem);

	//log data
	m_fixationsLog->Print(LRT_GAZEDATA);
	m_fixationsLog->Print(GDT_FIXATION);
	m_fixationsLog->Print(etiItem->Time());
	m_fixationsLog->Print(etiItem->Number());
	m_fixationsLog->Print(etiItem->Duration());
	m_fixationsLog->Print(etiItem->X());
	m_fixationsLog->Print(etiItem->Y());
	m_fixationsLog->Flush();
}

void CGazeDataClient::AddEyeSample(CGazeDataEntrySingleSamplePtr etiItem)
{
	//assume 0 = left eye, 1+ - right eye
	etiGazeSampleVector* eyeSamples = (etiItem->Eye() == 0)? &m_leftEyeSamples : &m_rightEyeSamples;

	while (eyeSamples->size() >= m_sampleQueueLength &&
		eyeSamples->size() > 0)
		eyeSamples->erase(eyeSamples->begin());

	eyeSamples->push_back(etiItem);

	//log samples
	m_singleSamplesLog->Print(LRT_GAZEDATA);
	m_singleSamplesLog->Print(GDT_SINGLESAMPLE);
	m_singleSamplesLog->Print(etiItem->Time());
	m_singleSamplesLog->Print(etiItem->Number());
	m_singleSamplesLog->Print(etiItem->X());
	m_singleSamplesLog->Print(etiItem->Y());
	m_singleSamplesLog->Print(etiItem->Eye());
	m_singleSamplesLog->Print(etiItem->Pupil());
	m_singleSamplesLog->Flush();
}

void CGazeDataClient::SetNick(std::string nick)
{
	this->m_nickname = nick;
}

void CGazeDataClient::SetDesc(std::string desc)
{
	this->m_description = desc;
}

std::string CGazeDataClient::GetNick()
{
	return this->m_nickname;
}

std::string CGazeDataClient::GetDesc()
{
	return this->m_description;
}

void CGazeDataClient::SetBlinksQueueLength(int length)
{
	m_blinkQueueLength = length;
}

void CGazeDataClient::SetFixationsQueueLength(int length)
{
	m_fixationQueueLength = length;
}

void CGazeDataClient::SetSamplesQueueLength(int length)
{
	m_sampleQueueLength = length;
}